/// <reference types="node" />
import { EventEmitter } from "events";
import IORedis = require("ioredis");
import { CancelResult, CheckResult, ReserveResult, SlidingWindowRateLimiterBackend, SlidingWindowRateLimiterBackendOptions } from "./sliding-window-rate-limiter-backend";
declare type Canceled = number;
declare type Reset = number;
declare type Token = number;
declare type Usage = number;
export interface Redis extends IORedis.Redis {
    limiter_cancel(key: string, token: number): Promise<Canceled>;
    limiter_check(key: string, interval: number, limit: number): Promise<[Usage, Reset]>;
    limiter_reserve(key: string, interval: number, limit: number): Promise<[Token, Usage, Reset]>;
}
export interface RedisSlidingWindowRateLimiterOptions extends SlidingWindowRateLimiterBackendOptions {
    redis?: Redis | string;
    interval?: number;
    operationTimeout?: number;
}
export declare class RedisSlidingWindowRateLimiter extends EventEmitter implements SlidingWindowRateLimiterBackend {
    readonly options: RedisSlidingWindowRateLimiterOptions;
    readonly redis: Redis;
    readonly interval: number;
    readonly operationTimeout: number;
    constructor(options?: RedisSlidingWindowRateLimiterOptions);
    cancel(key: string, token: number): Promise<CancelResult>;
    check(key: string, limit: number): Promise<CheckResult>;
    reserve(key: string, limit: number): Promise<ReserveResult>;
    destroy(): void;
    private promiseWithTimeoutError;
}
export default RedisSlidingWindowRateLimiter;
