import { RedisSlidingWindowRateLimiter, RedisSlidingWindowRateLimiterOptions } from "./redis-sliding-window-rate-limiter";
import { SlidingWindowRateLimiterBackend } from "./sliding-window-rate-limiter";
import { CancelResult, CheckResult, ReserveResult } from "./sliding-window-rate-limiter-backend";
export interface SafeRedisSlidingWindowRateLimiterOptions extends RedisSlidingWindowRateLimiterOptions {
    safe?: true;
    reuseRedisAfter?: number;
    defaultResponse?: number;
}
export interface SafeRedisSlidingWindowRateLimiter {
    addListener(event: "error", listener: (err: Error) => void): this;
    emit(event: "error", error: Error): boolean;
    listeners(event: "error"): Array<(err: Error) => void>;
    off(event: "error", listener: (err: Error) => void): this;
    on(event: "error", listener: (err: Error) => void): this;
    once(event: "error", listener: (err: Error) => void): this;
    prependListener(event: "error", listener: (err: Error) => void): this;
    prependOnceListener(event: "error", listener: (err: Error) => void): this;
    removeListener(event: "error", listener: (err: Error) => void): this;
}
export declare class SafeRedisSlidingWindowRateLimiter extends RedisSlidingWindowRateLimiter implements SlidingWindowRateLimiterBackend {
    readonly options: SafeRedisSlidingWindowRateLimiterOptions;
    readonly reuseRedisAfter: number;
    private redisServiceAvailable;
    private reconnectTimer?;
    private reconnectTimerStart?;
    constructor(options?: SafeRedisSlidingWindowRateLimiterOptions);
    cancel(key: string, token: number): Promise<CancelResult>;
    check(key: string, limit: number): Promise<CheckResult>;
    reserve(key: string, limit: number): Promise<ReserveResult>;
    destroy(): void;
    private errorHandler;
    private handleError;
    private markServiceAsUnavailable;
    private resultWithReset;
}
export default SafeRedisSlidingWindowRateLimiter;
