"use strict";
/// <reference types="node" />
Object.defineProperty(exports, "__esModule", { value: true });
exports.SafeRedisSlidingWindowRateLimiter = void 0;
const redis_sliding_window_rate_limiter_1 = require("./redis-sliding-window-rate-limiter");
class SafeRedisSlidingWindowRateLimiter extends redis_sliding_window_rate_limiter_1.RedisSlidingWindowRateLimiter {
    constructor(options = {}) {
        super(options);
        this.options = options;
        this.reuseRedisAfter = Number(this.options.reuseRedisAfter) || 2000;
        this.redisServiceAvailable = true;
        this.on("error", this.errorHandler);
    }
    cancel(key, token) {
        if (this.redisServiceAvailable) {
            try {
                return super.cancel(key, token).catch(err => {
                    this.handleError(err);
                    return { canceled: 0 };
                });
            }
            catch (e) {
                this.handleError(e);
            }
        }
        return Promise.resolve({ canceled: 0 });
    }
    async check(key, limit) {
        if (this.redisServiceAvailable) {
            try {
                return super.check(key, limit).catch(err => {
                    this.handleError(err);
                    return this.resultWithReset({ usage: 0 });
                });
            }
            catch (e) {
                this.handleError(e);
            }
        }
        return Promise.resolve(this.resultWithReset({ usage: 0 }));
    }
    async reserve(key, limit) {
        if (this.redisServiceAvailable) {
            try {
                return super.reserve(key, limit).catch(err => {
                    this.handleError(err);
                    return this.resultWithReset({ usage: 0 });
                });
            }
            catch (e) {
                this.handleError(e);
            }
        }
        return Promise.resolve(this.resultWithReset({ usage: 0 }));
    }
    destroy() {
        super.destroy();
        if (this.reconnectTimer) {
            clearTimeout(this.reconnectTimer);
        }
        this.removeAllListeners("error");
    }
    errorHandler(_error) {
        // ignore
    }
    handleError(error) {
        this.markServiceAsUnavailable();
        this.emit("error", error);
    }
    markServiceAsUnavailable() {
        this.redisServiceAvailable = false;
        if (this.reconnectTimer) {
            clearTimeout(this.reconnectTimer);
        }
        this.reconnectTimer = setTimeout(() => {
            this.redisServiceAvailable = true;
            this.reconnectTimerStart = undefined;
        }, this.reuseRedisAfter);
        this.reconnectTimerStart = new Date().getTime();
    }
    resultWithReset(result) {
        if (this.reconnectTimerStart) {
            const now = new Date().getTime();
            result.reset = this.reconnectTimerStart + this.reuseRedisAfter - now;
        }
        return result;
    }
}
exports.SafeRedisSlidingWindowRateLimiter = SafeRedisSlidingWindowRateLimiter;
exports.default = SafeRedisSlidingWindowRateLimiter;
//# sourceMappingURL=safe-redis-sliding-window-rate-limiter.js.map